#ifndef __EXT_HPP__
#define __EXT_HPP__

#define FIELD_SIZE  5
#define TARGET_X    4
#define TARGET_Y    3
#define DELAY       1

#define dir_modulus 15

typedef enum {
	north = 1, 
	east  = 2,
	south = 4,
	west  = 8
} dir_type;

typedef enum {
	safe = 0,
	hot = 1,
	burning = 2
} temperature_type;


#define NO_BORDER 0
#define NE_BORDER (north | east)
#define NW_BORDER (north | west)
#define SE_BORDER (south | east)
#define SW_BORDER (south | west)

const char* direction (int &dir);
const char* alarm (int &temperature);
int rotate_dir (int &dir);
int delta_x (int &dir);
int delta_y (int &dir);

typedef struct _Square {
	int timer;
	int temperature;
	int volatility;
	int border;
} Square;

class Field {
private:
	int delay;

	int target_x;
	int target_y;
	Square field[FIELD_SIZE][FIELD_SIZE];

	void init_square (int i, int j, temperature_type t);
	void evolve_square (int i, int j);

public:
	Field() {}

	void init_field(int t_x, int t_y, int delay); 
	void evolve_field();
	void print_field(int robot_x, int robot_y, int dir);
	
	int detect_target(int x, int y);
	int detect_borders(int x, int y);
	int temperature_in_front(int x, int y, int dir);	
};

void init_field (Field &field, int &t_x, int &t_y, int &delay);

int detect_target (Field &field, int &x, int &y);
int detect_borders(Field &field, int &x, int &y);
int temperature_in_front(Field &field, int &x, int &y, int &dir);

void init_field ();
int detect_target (int &x, int &y);
int detect_borders(int &x, int &y);
int temperature_in_front(int &x, int &y, int &dir);
void print_field (int &robot_x, int &robot_y, int &dir);
void evolve_field ();

#endif
